extern crate winapi;
use std::io::Error;

fn run_command(cmd: &str) -> Result<i32, Error> {
    use std::mem;
    use std::ffi::OsStr;
    use std::iter::once;
    use std::os::windows::ffi::OsStrExt;
    use std::ptr::null_mut;
    use winapi::shared::minwindef::{ BOOL, DWORD };
    use winapi::um::processthreadsapi::{ CreateProcessW, 
                                         STARTUPINFOW, 
                                         PROCESS_INFORMATION,
                                         GetExitCodeProcess };
    use winapi::um::synchapi::{ WaitForSingleObject };
    use winapi::um::winbase::{ INFINITE, WAIT_FAILED, CREATE_NEW_CONSOLE };

    let mut cmdline: Vec<u16> = OsStr::new(cmd).encode_wide().chain(once(0)).collect();
    let mut sinfo: STARTUPINFOW = unsafe { mem::zeroed() };
    sinfo.cb = mem::size_of::<STARTUPINFOW>() as DWORD;
    let mut pinfo: PROCESS_INFORMATION = unsafe { mem::zeroed() };

    let ret = unsafe {
        CreateProcessW( null_mut(), 
                        cmdline.as_mut_ptr(), 
                        null_mut(), 
                        null_mut(), 
                        false as BOOL,
                        CREATE_NEW_CONSOLE,
                        null_mut(),
                        null_mut(),
                        &mut sinfo,
                        &mut pinfo )
    };
    if ret == 0 { Err(Error::last_os_error()) }
    else        { Ok(ret) }.unwrap();

    println!("Waiting for process number {} to exit...", pinfo.dwProcessId);
    let ret = unsafe {
        WaitForSingleObject( pinfo.hProcess, 
                             INFINITE )
    };
    if ret == WAIT_FAILED { Err(Error::last_os_error()) }
    else { Ok(ret) }.unwrap();

    let mut exit_code = 0;
    let ret = unsafe {
        GetExitCodeProcess( pinfo.hProcess, 
                            &mut exit_code )
    };
    if ret == 0 { Err(Error::last_os_error()) }
    else        { Ok(exit_code as i32) }
}

fn main() {
    let exit_code = run_command("cmd.exe").unwrap();
    println!("cmd.exe exited with value {}", exit_code);
}

/*
CreateProcess( NULL, 
               lpCommandLine, 
               NULL, 
               NULL, 
               FALSE,
               CREATE_NEW_CONSOLE,
               NULL, 
               NULL, 
               &startInfo, 
               &processInfo )
*/
